@@ -1,4 +1,6 @@
 # encoding: utf-8
+require "yaml"
+
 module Mail
   # The Message class provides a single point of access to all things to do with an
   # email message.
@@ -115,7 +117,7 @@ module Mail
       @delivery_method = Mail.delivery_method.dup
 
       @transport_encoding = Mail::Encodings.get_encoding('7bit')
-      
+
       @mark_for_delete = false
 
       if args.flatten.first.respond_to?(:each_pair)
@@ -240,9 +242,9 @@ module Mail
     #
     # Returns self
     def deliver!
-      delivery_method.deliver!(self)
+      response = delivery_method.deliver!(self)
       inform_observers
-      self
+      delivery_method.settings[:return_response] ? response : self
     end
 
     def delivery_method(method = nil, settings = {})
@@ -259,14 +261,16 @@ module Mail
           bracketed_message_id = "<#{message_id}>"
           reply.in_reply_to = bracketed_message_id
           if !references.nil?
-            reply.references = (references.to_a.map { |r| "<#{r}>" } << bracketed_message_id).join(' ')
+            refs = [references].flatten.map { |r| "<#{r}>" }
+            refs << bracketed_message_id
+            reply.references = refs.join(' ')
           elsif !in_reply_to.nil? && !in_reply_to.kind_of?(Array)
             reply.references = "<#{in_reply_to}> #{bracketed_message_id}"
           end
           reply.references ||= bracketed_message_id
         end
         if subject
-          reply.subject = "RE: #{subject}"
+          reply.subject = subject =~ /^Re:/i ? subject : "RE: #{subject}"
         end
         if reply_to || from
           reply.to = self[reply_to ? :reply_to : :from].to_s
@@ -1675,6 +1679,7 @@ module Mail
       self.body = ''
       text_part = Mail::Part.new({:content_type => 'text/plain;',
                                   :body => text})
+      text_part.charset = charset unless @defaulted_charset
       self.body << text_part
     end
 
@@ -1706,6 +1711,25 @@ module Mail
       buffer
     end
 
+    def to_yaml
+      ready_to_send!
+      hash = {}
+      header.fields.each do |field|
+        hash[field.name] = field.value
+      end
+      hash['subject'] = subject
+      hash['body'] = body.encoded(content_transfer_encoding)
+      hash.to_yaml
+    end
+
+    def self.from_yaml(str)
+      from_hash(YAML::load(str))
+    end
+
+    def self.from_hash(hash)
+      Mail::Message.new(hash)
+    end
+
     def to_s
       encoded
     end
@@ -1758,9 +1782,9 @@ module Mail
     end
 
     def find_first_mime_type(mt)
-      all_parts.detect { |p| p.mime_type == mt }
+      all_parts.detect { |p| p.mime_type == mt && !p.attachment? }
     end
-  
+
     # Skips the deletion of this message. All other messages
     # flagged for delete still will be deleted at session close (i.e. when
     # #find exits). Only has an effect if you're using #find_and_delete
@@ -1768,28 +1792,28 @@ module Mail
     def skip_deletion
       @mark_for_delete = false
     end
-    
-    # Sets whether this message should be deleted at session close (i.e. 
+
+    # Sets whether this message should be deleted at session close (i.e.
     # after #find). Message will only be deleted if messages are retrieved
-    # using the #find_and_delete method, or by calling #find with 
-    # :delete_after_find set to true.    
+    # using the #find_and_delete method, or by calling #find with
+    # :delete_after_find set to true.
     def mark_for_delete=(value = true)
       @mark_for_delete = value
     end
-    
+
     # Returns whether message will be marked for deletion.
-    # If so, the message will be deleted at session close (i.e. after #find 
-    # exits), but only if also using the #find_and_delete method, or by 
+    # If so, the message will be deleted at session close (i.e. after #find
+    # exits), but only if also using the #find_and_delete method, or by
     # calling #find with :delete_after_find set to true.
     #
     # Side-note: Just to be clear, this method will return true even if
-    # the message hasn't yet been marked for delete on the mail server. 
+    # the message hasn't yet been marked for delete on the mail server.
     # However, if this method returns true, it *will be* marked on the
-    # server after each block yields back to #find or #find_and_delete. 
+    # server after each block yields back to #find or #find_and_delete.
     def is_marked_for_delete?
       return @mark_for_delete
     end
-    
+
   private
 
     #  2.1. General Description
@@ -1904,7 +1928,7 @@ module Mail
     end
 
     def init_with_hash(hash)
-      passed_in_options = hash.with_indifferent_access
+      passed_in_options = IndifferentHash.new(hash)
       self.raw_source = ''
 
       @header = Mail::Header.new
@@ -1955,6 +1979,7 @@ module Mail
       else
         filename = nil
       end
+      filename = Mail::Encodings.decode_encode(filename, :decode) if filename rescue filename
       filename
     end
 
